/*
 *    GeoTools - OpenSource mapping toolkit
 *    http://geotools.org
 *    (C) 2004-2006, GeoTools Project Managment Committee (PMC)
 *   
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 */
package org.geotools.referencing.factory.epsg;

// J2SE dependencies
import java.sql.Connection;
import java.util.Iterator;
import java.util.LinkedHashMap;
//import java.util.Map;
import java.util.HashMap;

import javax.units.NonSI;
import javax.units.Unit;
import javax.units.SI;


import javax.sql.DataSource;


import org.opengis.metadata.citation.Citation;
import org.opengis.referencing.FactoryException;


// Geotools dependencies
import org.geotools.factory.Hints;
import org.geotools.metadata.iso.citation.Citations;
import org.geotools.measure.Units;
import org.geotools.referencing.datum.BursaWolfParameters;
import org.geotools.resources.i18n.Errors;
import org.geotools.resources.i18n.ErrorKeys;

/**
 * An EPSG factory for the database generated by SQL scripts rather than the MS-Access one.
 * This class overrides {@link #adaptSQL} in order to translate SQL statements from
 * MS-Access syntax to ANSI syntax. By default, the translated SQL statements use the
 * table and field names in the Data Description Language (DDL) scripts provided by
 * EPSG to create the schema for the database. Subclasses can changes this default
 * behavior by modifying the {@link #map}.
 * <p>
 * <strong>References:</strong><ul>
 *   <li>EPSG geodecy parameters database readme at
 *       <A HREF="http://www.ihsenergy.com/epsg/geodetic2.html">www.epsg.org</A>
 *   </li>
 * </ul>
 *
 * @since 2.4
 * @source $URL: http://svn.geotools.org/geotools/branches/2.4.x/modules/library/referencing/src/main/java/org/geotools/referencing/factory/epsg/AnsiDialectEpsgFactory.java $
 * @version $Id: AnsiDialectEpsgFactory.java 26328 2007-07-24 16:57:19Z desruisseaux $
 * @author Rueben Schulz
 * @author Martin Desruisseaux
 * @author Didier Richard
 * @author John Grange
 */

public class AnsiDialectIgnfFactory extends AnsiDialectEpsgFactory {

    public AnsiDialectIgnfFactory(final Hints      userHints,
                                  final Connection connection)
    {
        super(userHints, connection);
        setSchema("ignf_");
    }

    public synchronized Citation getAuthority() {
        return Citations.IGNF; 
    }


    /*
     * A IGNF primary key is a string that contains only
     *  - Uppercase latin letters 
     *  - digits (0 - 9)
     *  - zero or one dot '.'
     * The following filter returns true for a real primary key.
     * It may however retrun true for other string that are not primary key 
     * 
     * This function overrides the function of the DirectEpsgFactory class
     * that was scanning for epsg numerical codes.   
     *
     * TODO: FIXME: Enlever les "_" dans la base !!!!
     */

    protected boolean isPrimaryKey(final String code) {
        final int length = code.length();
        boolean dot = false;
        boolean underscore = false;
        for (int i=0; i<length; i++) {         
          final char c = code.charAt(i);
          if (c == '.') {
            if(dot) return false;          // there is at most one dot in a IGNF code
            dot = true;
          }
          else if (c == '_') {
            if(underscore) return false;          // there is at most one dot in a IGNF code
            underscore = true;
          }
          else if(!((c >= 'A' && c <= 'Z') || (c >= '0' && c <= '9'))) return false;
        }
        return true;
    }

    protected boolean isBursaWolfCode(String code) {
      return code.equals("TSGM110") || code.equals("TSGM120");
    }

    protected boolean isRotationFrameCode(String code) {
      return false;
    }

    protected String getSQLBursaWolfParameterFilter() {
      return "(CO.COORD_OP_METHOD_CODE = 'TSGM110' OR CO.COORD_OP_METHOD_CODE = 'TSGM120')";
    }

  /**
   *
   *
   */ 
    private static HashMap unitMap = null;   
    protected Unit getUnit(final String code) {
      if(unitMap == null) {
        synchronized(this.getClass()) {
          if(unitMap == null) {
            unitMap = new HashMap(32);
            unitMap.put("ANG01", SI.RADIAN);
            unitMap.put("ANG02", NonSI.DEGREE_ANGLE);
        //  unitMap.put("ANG03", NonSI.GRADE);
            unitMap.put("ANG05", NonSI.SECOND_ANGLE);
            unitMap.put("UNL01", SI.METER);
            unitMap.put("SCA01", Unit.ONE);
            unitMap.put("SCA02", Units.PPM);        
          }
        }
      }
      return (Unit) unitMap.get(code);
    }


    /**
     * Overload the function 
     *
     *
     */

    protected void setBursaWolfParameter(final BursaWolfParameters parameters,
                                   final String code, double value, final Unit unit) 
     throws FactoryException
     {
        Unit target = unit;

        if(code.equals("TSGP101")) target = SI.METER;
        else if(code.equals("TSGP102")) target = SI.METER;
        else if(code.equals("TSGP103")) target = SI.METER;
        else if(code.equals("TSGP104")) target = Units.PPM;
        else if(code.equals("TSGP105")) target = NonSI.SECOND_ANGLE;
        else if(code.equals("TSGP106")) target = NonSI.SECOND_ANGLE;
        else if(code.equals("TSGP107")) target = NonSI.SECOND_ANGLE;
        else throw new FactoryException(Errors.format(ErrorKeys.UNEXPECTED_PARAMETER_$1, code));
  
        if (target != unit) {
            value = unit.getConverterTo(target).convert(value);
        }

        if(code.equals("TSGP101")) parameters.dx  = value;
        else if(code.equals("TSGP102")) parameters.dy  = value;
        else if(code.equals("TSGP103")) parameters.dz  = value;
        else if(code.equals("TSGP104")) parameters.ppm = value;
        else if(code.equals("TSGP105")) parameters.ex  = value;
        else if(code.equals("TSGP106")) parameters.ey  = value;
        else if(code.equals("TSGP107")) parameters.ez  = value;
    }
}
