/*
 *    GeoTools - OpenSource mapping toolkit
 *    http://geotools.org
 *    (C) 2005-2006, GeoTools Project Managment Committee (PMC)
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 */
package org.geotools.referencing.factory.epsg;

// J2SE dependencies
import java.sql.Connection;
import java.sql.SQLException;

import org.geotools.factory.Hints;
import org.geotools.metadata.iso.citation.Citations;
import org.geotools.referencing.factory.AbstractAuthorityFactory;
import org.geotools.resources.i18n.ErrorKeys;
import org.geotools.resources.i18n.Errors;
import org.geotools.util.GenericName;
import org.opengis.metadata.citation.Citation;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

/**
 * IGNFPATCH
 * Extension of the connection to the IGNF database in PostgreSQL database
 * engine using JDBC. The EPSG database can be downloaded from <A
 * HREF="http://www.epsg.org">http://www.epsg.org</A>. It should have been
 * imported into a PostgreSQL database, which doesn't need to be on the local
 * machine.
 * <p>
 * <h3>Connection parameters</h3>
 * The preferred way to specify connection parameters is through the JNDI
 * interface. However, this datasource provides the following alternative as a
 * convenience: if a {@value #CONFIGURATION_FILE} file is found in current
 * directory or in the user's home directory, then the following properties are
 * fetch. Note that the default value may change in a future version if a public
 * server become available.
 * <P>
 * <TABLE BORDER="1">
 * <TR>
 * <TH>Property</TH>
 * <TH>Type</TH>
 * <TH>Description</TH>
 * <TH>Geotools Default</TH>
 * </TR>
 * <TR>
 * <TD>{@code serverName}</TD>
 * <TD>String</TD>
 * <TD>PostgreSQL database server host name</TD>
 * <TD>{@code localhost}</TD>
 * </TR>
 * <TR>
 * <TD>{@code databaseName}</TD>
 * <TD>String</TD>
 * <TD>PostgreSQL database name</TD>
 * <TD>{@code EPSG}</TD>
 * </TR>
 * <TR>
 * <TD>{@code schema}</TD>
 * <TD>String</TD>
 * <TD>The schema for the EPSG tables</TD>
 * <TD></TD>
 * </TR>
 * <TR>
 * <TD>{@code portNumber}</TD>
 * <TD>int</TD>
 * <TD>TCP port which the PostgreSQL database server is listening on</TD>
 * <TD>{@code 5432}</TD>
 * </TR>
 * <TR>
 * <TD>{@code user}</TD>
 * <TD>String</TD>
 * <TD>User used to make database connections</TD>
 * <TD>{@code GeoTools}</TD>
 * </TR>
 * <TR>
 * <TD>{@code password}</TD>
 * <TD>String</TD>
 * <TD>Password used to make database connections</TD>
 * <TD>{@code GeoTools}</TD>
 * </TR>
 * </TABLE>
 * <P>
 * The database version is given in the
 * {@linkplain org.opengis.metadata.citation.Citation#getEdition edition attribute}
 * of the
 * {@linkplain org.opengis.referencing.AuthorityFactory#getAuthority authority}.
 * The postgreSQL database should be read only.
 * <P>
 * Just having this class accessible in the classpath, together with the
 * registration in the {@code META-INF/services/} directory, is suffisient to
 * get a working EPSG authority factory backed by this database. Vendors can
 * create a copy of this class, modify it and bundle it with their own
 * distribution if they want to connect their users to an other database.
 * 
 * @since 2.4
 * @source $URL:
 *         http://svn.geotools.org/geotools/branches/2.4.x/modules/plugin/epsg-postgresql/src/main/java/org/geotools/referencing/factory/epsg/ThreadedPostgreSQLEpsgFactory.java $
 * @version $Id: ThreadedPostgreSQLEpsgFactory.java 26096 2007-06-29 21:45:12Z
 *          jgarnett $
 * @author Didier Richard
 * @author Martin Desruisseaux
 * 
 * @tutorial http://docs.codehaus.org/display/GEOTOOLS/How+to+install+the+EPSG+database+in+PostgreSQL
 */
public class HTTPThreadedPostgreSQLIgnfFactory extends
		ThreadedPostgreSQLIgnfFactory {

	/**
	 * The base URL, which is {@value}.
	 */
	public static final String BASE_URL = "http://librairies.ign.fr/geoportail/resources/RIG.xml#";

	/**
	 * Constructors
	 */
	public HTTPThreadedPostgreSQLIgnfFactory() {
		this(null);
	}

	public HTTPThreadedPostgreSQLIgnfFactory(final Hints userHints) {
		super(userHints);
	}

	/**
	 * The schema name, or {@code null} if none.
	 */
	private String schema;

	public synchronized Citation getAuthority() {
		return Citations.HTTP_IGNF;
	}

	/**
	 * Removes the URL base ({@value #BASE_URL}) from the specified code
	 * before to pass it to the wrapped factories.
	 * 
	 * @param code
	 *            The code given to this factory.
	 * @return The code to give to the underlying factories.
	 * @throws FactoryException
	 *             if the code can't be converted.
	 */
	protected String toBackingFactoryCode(String code) throws FactoryException {
		code = code.trim();
		final int length = BASE_URL.length();
		if (code.regionMatches(true, 0, BASE_URL, 0, length)) {
			code = code.substring(length);
			return code;
		}

		throw new NoSuchAuthorityCodeException(Errors.format(
				ErrorKeys.ILLEGAL_ARGUMENT_$2, "code", code), BASE_URL, code);
	}

	/**
	 * Returns the backing-store factory for PostgreSQL syntax.
	 * 
	 * @param hints
	 *            A map of hints, including the low-level factories to use for
	 *            CRS creation.
	 * @return The EPSG factory using PostgreSQL syntax.
	 * @throws SQLException
	 *             if connection to the database failed.
	 */
	protected AbstractAuthorityFactory createBackingStore(final Hints hints)
			throws SQLException {
		final Connection connection = getDataSource().getConnection();
		final HTTPAnsiDialectIgnfFactory factory = new HTTPAnsiDialectIgnfFactory(
				hints, connection);
		if (schema != null) {
			factory.setSchema(schema);
		}
		return factory;
	}

}
